@@ -15,7 +15,7 @@ our @EXPORT_OK = (
 );
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
 
-our $VERSION = "v6.23.0";
+our $VERSION = "v6.24.0";
 
 1;
 __END__
@@ -61,7 +61,7 @@ The documentation in this POD applies to all three adapter modules as well.
 This module is an implementation of L<Reactive Extensions|http://reactivex.io/> in Perl. It replicates the
 behavior of L<rxjs 6|https://www.npmjs.com/package/rxjs> which is the JavaScript implementation of ReactiveX.
 
-Currently 93 of the 100+ operators in rxjs are implemented in this module.
+Currently 94 of the 100+ operators in rxjs are implemented in this module.
 
 =head1 EXPORTABLE FUNCTIONS
 
@@ -1081,6 +1081,25 @@ L<https://rxjs.dev/api/operators/share>
     $o->subscribe($observer1);
     $o->subscribe($observer2);
 
+=item op_single
+
+L<https://rxjs.dev/api/operators/single>
+
+    # error: Too many values match
+    rx_of(0, 1, 2, 3)->pipe(
+        op_single(sub ($val, $idx) { $val % 2 == 1 }), # can also use $_ here
+    )->subscribe($observer);
+
+    # error: No values match
+    rx_of(1, 3)->pipe(
+        op_single(sub { $_ % 2 == 0 }),
+    )->subscribe($observer);
+
+    # 42, complete
+    rx_of(42)->pipe(
+        op_single(),
+    )->subscribe($observer);
+
 =item op_skip
 
 L<https://rxjs.dev/api/operators/skip>
