@@ -21,13 +21,13 @@ our @EXPORT_OK = qw/
     op_exhaust_map op_filter op_finalize op_find op_find_index op_first op_group_by op_ignore_elements op_is_empty
     op_last op_map op_map_to op_max op_merge_all op_merge_map op_merge_with op_min op_multicast
     op_on_error_resume_next_with op_pairwise op_pluck op_race_with op_reduce op_ref_count op_repeat op_retry op_sample
-    op_sample_time op_scan op_share op_skip op_skip_until op_skip_while op_start_with op_switch_all op_switch_map
-    op_take op_take_last op_take_until op_take_while op_tap op_throttle op_throttle_time op_throw_if_empty op_to_array
-    op_with_latest_from op_zip_with
+    op_sample_time op_scan op_share op_single op_skip op_skip_until op_skip_while op_start_with op_switch_all
+    op_switch_map op_take op_take_last op_take_until op_take_while op_tap op_throttle op_throttle_time
+    op_throw_if_empty op_to_array op_with_latest_from op_zip_with
 /;
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
 
-our $VERSION = "v6.23.0";
+our $VERSION = "v6.24.0";
 
 sub op_audit {
     my ($duration_selector) = @_;
@@ -1057,6 +1057,38 @@ sub op_max {
     };
 }
 
+sub _op_merge_all_make_subscriber {
+    my ($small_subscriptions, $subscriber, $stored_observables, $big_completed_ref) = @_;
+
+    my $small_subscription = RxPerl::Subscription->new;
+    $small_subscriptions->{$small_subscription} = $small_subscription;
+    return {
+        new_subscription => $small_subscription,
+        next             => sub {
+            $subscriber->{next}->(@_) if defined $subscriber->{next};
+        },
+        error            => sub {
+            $subscriber->{error}->(@_) if defined $subscriber->{error};
+        },
+        complete         => sub {
+            delete $small_subscriptions->{$small_subscription};
+            if (@$stored_observables) {
+                my $new_obs = shift @$stored_observables;
+                $new_obs->subscribe(
+                    _op_merge_all_make_subscriber(
+                        $small_subscriptions,
+                        $subscriber,
+                        $stored_observables,
+                        $big_completed_ref,
+                    ),
+                );
+            } elsif ($$big_completed_ref and !%$small_subscriptions) {
+                $subscriber->{complete}->() if defined $subscriber->{complete};
+            }
+        },
+    };
+}
+
 sub op_merge_all {
     my ($concurrent) = @_;
 
@@ -1066,9 +1098,9 @@ sub op_merge_all {
         return rx_observable->new(sub {
             my ($subscriber) = @_;
 
-            my $num_subscriptions = 0;
             my @stored_observables;
             my %small_subscriptions;
+            my $big_completed;
 
             my $own_subscription = RxPerl::Subscription->new;
             $subscriber->subscription->add(
@@ -1076,50 +1108,30 @@ sub op_merge_all {
                 \%small_subscriptions,
             );
 
-            my $weak_sub;
-            my $make_subscriber_sub = sub {
-                my $small_subscription = RxPerl::Subscription->new;
-                $small_subscriptions{$small_subscription} = $small_subscription;
-                return {
-                    new_subscription => $small_subscription,
-                    next     => sub {
-                        $subscriber->{next}->(@_) if defined $subscriber->{next};
-                    },
-                    error    => sub {
-                        $subscriber->{error}->(@_) if defined $subscriber->{error};
-                    },
-                    complete => sub {
-                        $num_subscriptions--;
-                        delete $small_subscriptions{$small_subscription};
-                        if (@stored_observables) {
-                            $num_subscriptions++;
-                            my $new_obs = shift @stored_observables;
-                            $new_obs->subscribe($weak_sub->());
-                        } else {
-                            $subscriber->{complete}->() if !$num_subscriptions and defined $subscriber->{complete};
-                        }
-                    },
-                };
-            };
-            weaken($weak_sub = $make_subscriber_sub);
-
             my $own_subscriber = {
                 new_subscription => $own_subscription,
-                next     => sub {
+                next             => sub {
                     my ($new_observable) = @_;
 
                     push @stored_observables, $new_observable;
-                    if (! defined $concurrent or $num_subscriptions < $concurrent) {
-                        $num_subscriptions++;
+                    if (!defined $concurrent or keys(%small_subscriptions) < $concurrent) {
                         my $new_obs = shift @stored_observables;
-                        $new_obs->subscribe($make_subscriber_sub->());
+                        $new_obs->subscribe(
+                            _op_merge_all_make_subscriber(
+                                \%small_subscriptions,
+                                $subscriber,
+                                \@stored_observables,
+                                \$big_completed,
+                            ),
+                        );
                     }
                 },
-                error    => sub {
+                error            => sub {
                     $subscriber->{error}->(@_) if defined $subscriber->{error};
                 },
-                complete => sub {
-                    $subscriber->{complete}->() if !$num_subscriptions and defined $subscriber->{complete};
+                complete         => sub {
+                    $big_completed = 1;
+                    $subscriber->{complete}->() if !%small_subscriptions and defined $subscriber->{complete};
                 },
             };
 
@@ -1600,6 +1612,54 @@ sub op_share {
     );
 }
 
+sub op_single {
+    my ($predicate) = @_;
+
+    return sub {
+        my ($source) = @_;
+
+        return rx_observable->new(sub {
+            my ($subscriber) = @_;
+
+            my @found;
+
+            my $idx = 0;
+            my $own_subscriber = {
+                %$subscriber,
+                next     => sub {
+                    my ($v) = @_;
+
+                    if (!$predicate) {
+                        push @found, $v;
+                    } else {
+                        my $found;
+                        my $ok = eval { local $_ = $v; $found = $predicate->($v, $idx++); 1 };
+                        if (! $ok) {
+                            $subscriber->{error}->($@) if defined $subscriber->{error};
+                            return;
+                        }
+                        push @found, $v if $found;
+                    }
+
+                    $subscriber->{error}->('Too many values match') if @found > 1 and defined $subscriber->{error};
+                },
+                complete => sub {
+                    if (! @found) {
+                        $subscriber->{error}->('No values match') if defined $subscriber->{error};
+                    } else {
+                        $subscriber->{next}->($found[0]) if defined $subscriber->{next};
+                        $subscriber->{complete}->() if defined $subscriber->{complete};
+                    }
+                },
+            };
+
+            $source->subscribe($own_subscriber);
+
+            return;
+        });
+    };
+}
+
 sub op_skip {
     my ($count) = @_;
 
